function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { createStore, registerBridgeStore } from '@segment/sovran-react-native';
import deepmerge from 'deepmerge';
import { getUUID } from '../uuid';
import { createGetter } from './helpers';
// NOTE: Not exported from @segment/sovran-react-native. Must explicitly declare here.
// Also this fallback is used in store.ts in @segment/sovran-react-native yet "storeId" is required.
const DEFAULT_STORE_NAME = 'default';
const INITIAL_VALUES = {
  isReady: true,
  events: [],
  eventsToRetry: [],
  context: {},
  settings: {},
  userInfo: {
    anonymousId: getUUID(),
    userId: undefined,
    traits: undefined
  }
};

/**
 * Global store for deeplink information
 * A single instance is needed for all SovranStorage objects since only one deeplink data exists at a time
 * No need to persist this information
 */
const deepLinkStore = createStore({
  referring_application: '',
  url: ''
});
/**
 * Action to set the referring app and link url
 * @param deepLinkData referring app and link url
 */

const addDeepLinkData = deepLinkData => () => {
  return {
    referring_application: deepLinkData.referring_application,
    url: deepLinkData.url
  };
};
/**
 * Registers the deeplink store to listen to native events
 */


registerBridgeStore({
  store: deepLinkStore,
  actions: {
    'add-deepLink-data': addDeepLinkData
  }
});

function createStoreGetter(store, key) {
  return createGetter(() => {
    const state = store.getState();

    if (key !== undefined) {
      return state[key];
    }

    return state;
  }, async () => {
    const promise = await store.getState(true);

    if (key !== undefined) {
      return promise[key];
    }

    return promise;
  });
}

export class SovranStorage {
  constructor(config) {
    _defineProperty(this, "storeId", void 0);

    _defineProperty(this, "storePersistor", void 0);

    _defineProperty(this, "readinessStore", void 0);

    _defineProperty(this, "contextStore", void 0);

    _defineProperty(this, "settingsStore", void 0);

    _defineProperty(this, "userInfoStore", void 0);

    _defineProperty(this, "deepLinkStore", deepLinkStore);

    _defineProperty(this, "isReady", void 0);

    _defineProperty(this, "context", void 0);

    _defineProperty(this, "settings", void 0);

    _defineProperty(this, "userInfo", void 0);

    _defineProperty(this, "deepLinkData", void 0);

    _defineProperty(this, "fixAnonymousId", () => {
      const fixUnsubscribe = this.userInfoStore.subscribe(store => {
        if (store.userInfo.anonymousId === 'anonymousId') {
          this.userInfoStore.dispatch(state => {
            return {
              userInfo: { ...state.userInfo,
                anonymousId: getUUID()
              }
            };
          });
        }

        fixUnsubscribe();
      });
    });

    this.storeId = config.storeId;
    this.storePersistor = config.storePersistor;
    this.readinessStore = createStore({
      hasLoadedContext: false
    }, {
      persist: {
        storeId: DEFAULT_STORE_NAME,
        persistor: this.storePersistor
      }
    });
    this.isReady = {
      get: createStoreGetter(this.readinessStore, 'hasLoadedContext'),
      onChange: callback => {
        return this.readinessStore.subscribe(store => {
          if (store.hasLoadedContext) {
            callback(true);
          }
        });
      }
    };
    this.contextStore = createStore({
      context: INITIAL_VALUES.context
    }, {
      persist: {
        storeId: `${this.storeId}-context`,
        persistor: this.storePersistor
      }
    });
    this.context = {
      get: createStoreGetter(this.contextStore, 'context'),
      onChange: callback => this.contextStore.subscribe(store => callback(store.context)),
      set: async value => {
        const {
          context
        } = await this.contextStore.dispatch(state => {
          let newState;

          if (value instanceof Function) {
            newState = value(state.context);
          } else {
            newState = deepmerge(state.context, value);
          }

          return {
            context: newState
          };
        });
        return context;
      }
    };
    this.settingsStore = createStore({
      settings: INITIAL_VALUES.settings
    }, {
      persist: {
        storeId: `${this.storeId}-settings`,
        persistor: this.storePersistor
      }
    });
    this.settings = {
      get: createStoreGetter(this.settingsStore, 'settings'),
      onChange: callback => this.settingsStore.subscribe(store => callback(store.settings)),
      set: async value => {
        const {
          settings
        } = await this.settingsStore.dispatch(state => {
          let newState;

          if (value instanceof Function) {
            newState = value(state.settings);
          } else {
            newState = { ...state.settings,
              ...value
            };
          }

          return {
            settings: newState
          };
        });
        return settings;
      },
      add: (key, value) => {
        this.settingsStore.dispatch(state => ({
          settings: { ...state.settings,
            [key]: value
          }
        }));
      }
    };
    this.userInfoStore = createStore({
      userInfo: INITIAL_VALUES.userInfo
    }, {
      persist: {
        storeId: `${this.storeId}-userInfo`,
        persistor: this.storePersistor
      }
    });
    this.userInfo = {
      get: createStoreGetter(this.userInfoStore, 'userInfo'),
      onChange: callback => this.userInfoStore.subscribe(store => callback(store.userInfo)),
      set: async value => {
        const {
          userInfo
        } = await this.userInfoStore.dispatch(state => {
          let newState;

          if (value instanceof Function) {
            newState = value(state.userInfo);
          } else {
            newState = deepmerge(state.userInfo, value);
          }

          return {
            userInfo: newState
          };
        });
        return userInfo;
      }
    };
    this.deepLinkData = {
      get: createStoreGetter(this.deepLinkStore),
      onChange: callback => this.deepLinkStore.subscribe(callback)
    };
    this.fixAnonymousId(); // Wait for context to be loaded

    const unsubscribeContext = this.contextStore.subscribe(store => {
      if (store.context !== INITIAL_VALUES.context) {
        this.readinessStore.dispatch(state => ({ ...state,
          hasLoadedContext: true
        }));
        unsubscribeContext();
      }
    });
  }
  /**
   * This is a fix for users that have started the app with the anonymousId set to 'anonymousId' bug
   */


}
//# sourceMappingURL=sovranStorage.js.map